@@ -56,6 +56,9 @@ DOCUMENTATION = """
         version_added: '2.12'
         type: str
         default: '#}'
+    seealso:
+      - ref: playbook_task_paths
+        description: Search paths used for relative templates.
 """
 
 EXAMPLES = """
@@ -86,7 +89,7 @@ import ansible.constants as C
 
 from ansible.errors import AnsibleError
 from ansible.plugins.lookup import LookupBase
-from ansible.module_utils._text import to_text
+from ansible.module_utils.common.text.converters import to_text
 from ansible.template import generate_ansible_template_vars, AnsibleEnvironment
 from ansible.utils.display import Display
 from ansible.utils.native_jinja import NativeJinjaText
