@@ -24,7 +24,7 @@ from functools import wraps
 
 from ansible.errors import AnsibleError
 from ansible.plugins import AnsiblePlugin
-from ansible.module_utils._text import to_native
+from ansible.module_utils.common.text.converters import to_native
 from ansible.module_utils.basic import missing_required_lib
 
 try:
@@ -32,7 +32,10 @@ try:
     from ncclient.xml_ import to_xml, to_ele, NCElement
     HAS_NCCLIENT = True
     NCCLIENT_IMP_ERR = None
-except (ImportError, AttributeError) as err:  # paramiko and gssapi are incompatible and raise AttributeError not ImportError
+# paramiko and gssapi are incompatible and raise AttributeError not ImportError
+# When running in FIPS mode, cryptography raises InternalError
+# https://bugzilla.redhat.com/show_bug.cgi?id=1778939
+except Exception as err:
     HAS_NCCLIENT = False
     NCCLIENT_IMP_ERR = err
 
@@ -59,8 +62,8 @@ class NetconfBase(AnsiblePlugin):
         :class:`TerminalBase` plugins are byte strings.  This is because of
         how close to the underlying platform these plugins operate.  Remember
         to mark literal strings as byte string (``b"string"``) and to use
-        :func:`~ansible.module_utils._text.to_bytes` and
-        :func:`~ansible.module_utils._text.to_text` to avoid unexpected
+        :func:`~ansible.module_utils.common.text.converters.to_bytes` and
+        :func:`~ansible.module_utils.common.text.converters.to_text` to avoid unexpected
         problems.
 
         List of supported rpc's:
@@ -81,7 +84,7 @@ class NetconfBase(AnsiblePlugin):
             :execute_rpc: RPC to be execute on remote device
             :load_configuration: Loads given configuration on device
 
-        Note: rpc support depends on the capabilites of remote device.
+        Note: rpc support depends on the capabilities of remote device.
 
         :returns: Returns output received from remote device as byte string
         Note: the 'result' or 'error' from response should to be converted to object
