@@ -297,7 +297,7 @@ def run_pip(
                 connection.run([python.path], data=script, capture=True)
             except SubprocessError as ex:
                 if 'pip is unavailable:' in ex.stdout + ex.stderr:
-                    raise PipUnavailableError(python)
+                    raise PipUnavailableError(python) from None
 
             raise
 
@@ -441,8 +441,8 @@ def get_venv_packages(python: PythonConfig) -> dict[str, str]:
     #       See: https://github.com/ansible/base-test-container/blob/main/files/installer.py
 
     default_packages = dict(
-        pip='21.3.1',
-        setuptools='60.8.2',
+        pip='23.1.2',
+        setuptools='67.7.2',
         wheel='0.37.1',
     )
 
@@ -452,11 +452,6 @@ def get_venv_packages(python: PythonConfig) -> dict[str, str]:
             setuptools='44.1.1',  # 45.0.0 requires Python 3.5+
             wheel=None,
         ),
-        '3.5': dict(
-            pip='20.3.4',  # 21.0 requires Python 3.6+
-            setuptools='50.3.2',  # 51.0.0 requires Python 3.6+
-            wheel=None,
-        ),
         '3.6': dict(
             pip='21.3.1',  # 22.0 requires Python 3.7+
             setuptools='59.6.0',  # 59.7.0 requires Python 3.7+
