@@ -53,7 +53,7 @@ install_pip() {
                 pip_bootstrap_url="https://ci-files.testing.ansible.com/ansible-test/get-pip-20.3.4.py"
                 ;;
             *)
-                pip_bootstrap_url="https://ci-files.testing.ansible.com/ansible-test/get-pip-21.3.1.py"
+                pip_bootstrap_url="https://ci-files.testing.ansible.com/ansible-test/get-pip-23.1.2.py"
                 ;;
         esac
 
@@ -163,8 +163,6 @@ bootstrap_remote_freebsd()
         # Declare platform/python version combinations which do not have supporting OS packages available.
         # For these combinations ansible-test will use pip to install the requirements instead.
         case "${platform_version}/${python_version}" in
-            "12.4/3.9")
-                ;;
             *)
                 jinja2_pkg=""  # not available
                 cryptography_pkg=""  # not available
@@ -261,7 +259,7 @@ bootstrap_remote_rhel_8()
     if [ "${python_version}" = "3.6" ]; then
         py_pkg_prefix="python3"
     else
-        py_pkg_prefix="python${python_package_version}"
+        py_pkg_prefix="python${python_version}"
     fi
 
     packages="
@@ -269,6 +267,14 @@ bootstrap_remote_rhel_8()
         ${py_pkg_prefix}-devel
         "
 
+    # pip isn't included in the Python devel package under Python 3.11
+    if [ "${python_version}" != "3.6" ]; then
+        packages="
+            ${packages}
+            ${py_pkg_prefix}-pip
+        "
+    fi
+
     # Jinja2 is not installed with an OS package since the provided version is too old.
     # Instead, ansible-test will install it using pip.
     if [ "${controller}" ]; then
@@ -278,9 +284,19 @@ bootstrap_remote_rhel_8()
             "
     fi
 
+    # Python 3.11 isn't a module like the earlier versions
+    if [ "${python_version}" = "3.6" ]; then
+        while true; do
+            # shellcheck disable=SC2086
+            yum module install -q -y "python${python_package_version}" \
+            && break
+            echo "Failed to install packages. Sleeping before trying again..."
+            sleep 10
+        done
+    fi
+
     while true; do
         # shellcheck disable=SC2086
-        yum module install -q -y "python${python_package_version}" && \
         yum install -q -y ${packages} \
         && break
         echo "Failed to install packages. Sleeping before trying again..."
@@ -292,22 +308,34 @@ bootstrap_remote_rhel_8()
 
 bootstrap_remote_rhel_9()
 {
-    py_pkg_prefix="python3"
+    if [ "${python_version}" = "3.9" ]; then
+        py_pkg_prefix="python3"
+    else
+        py_pkg_prefix="python${python_version}"
+    fi
 
     packages="
         gcc
         ${py_pkg_prefix}-devel
         "
 
+    # pip is not included in the Python devel package under Python 3.11
+    if [ "${python_version}" != "3.9" ]; then
+        packages="
+            ${packages}
+            ${py_pkg_prefix}-pip
+        "
+    fi
+
     # Jinja2 is not installed with an OS package since the provided version is too old.
     # Instead, ansible-test will install it using pip.
+    # packaging and resolvelib are missing for Python 3.11 (and possible later) so we just
+    # skip them and let ansible-test install them from PyPI.
     if [ "${controller}" ]; then
         packages="
             ${packages}
             ${py_pkg_prefix}-cryptography
-            ${py_pkg_prefix}-packaging
             ${py_pkg_prefix}-pyyaml
-            ${py_pkg_prefix}-resolvelib
             "
     fi
 
@@ -387,14 +415,6 @@ bootstrap_remote_ubuntu()
         echo "Failed to install packages. Sleeping before trying again..."
         sleep 10
     done
-
-    if [ "${controller}" ]; then
-        if [ "${platform_version}/${python_version}" = "20.04/3.9" ]; then
-            # Install pyyaml using pip so libyaml support is available on Python 3.9.
-            # The OS package install (which is installed by default) only has a .so file for Python 3.8.
-            pip_install "--upgrade pyyaml"
-        fi
-    fi
 }
 
 bootstrap_docker()
